function define(name, dependencies, handler){
  var exports = {}
  handler(function(){}, exports)
  for(var i in exports){
    window[i] = exports[i]
  }
}

module.exports = {
    src: [
        './src',
    ],
    mode: 'file',
    includeDeclarations: true,
    tsconfig: 'tsconfig.json',
    out: './docs',
    excludePrivate: true,
    excludeProtected: true,
    excludeExternals: true,
    readme: 'README.md',
    name: 'plugin-container',
    ignoreCompilerErrors: true
};
define("src/index", ["require", "exports"], function (require, exports) {
    "use strict";
    exports.__esModule = true;
    var PluginContainer = (function () {
        function PluginContainer() {
            this.plugins = [];
            this.initialize();
        }
        PluginContainer.prototype.initialize = function () {
            this.plugins = [];
        };
        PluginContainer.prototype.executeAll = function (input) {
            var args = Array.prototype.slice.call(arguments, 0);
            this.plugins.forEach(function (p) {
                args[0] = p.execute.apply(p, args) || args[0];
            });
            return args[0];
        };
        PluginContainer.prototype.executeAllWithContext = function (context) {
            var theContext = arguments[0];
            var args = Array.prototype.slice.call(arguments, 1);
            this.plugins.forEach(function (p) {
                args[0] = p.execute.apply(theContext || p, args) || args[0];
            });
            return args[0];
        };
        PluginContainer.prototype.install = function (plugin) {
            plugin.priority = plugin.priority || PluginContainer.DEFAULT_PRIORITY;
            this.plugins.push(plugin);
            this.plugins = this.plugins.sort(function (a, b) { return (a.priority < b.priority ? 1 : -1); });
        };
        PluginContainer.prototype.uninstall = function (plugin) {
            var name = this.getPluginName(plugin);
            this.plugins = this.plugins.filter(function (p) { return p.name !== name; });
        };
        PluginContainer.prototype.getPluginName = function (plugin) {
            return typeof plugin === 'string' ? plugin : plugin.name;
        };
        PluginContainer.DEFAULT_PRIORITY = 5;
        return PluginContainer;
    }());
    exports.PluginContainer = PluginContainer;
});
define("spec/errorsSpec", ["require", "exports", "src/index"], function (require, exports, index_1) {
    "use strict";
    exports.__esModule = true;
    xdescribe('errors in plugins', function () {
        var container;
        beforeEach(function () {
            container = new index_1.PluginContainer();
        });
        it('plugin throw exception shouldallow the rest of plugins to execute, by default', function () {
            container.install({
                name: 'thrower plugin',
                execute: function (input) {
                    throw Error('thrower!');
                }
            });
            expect(1).toBe(1);
        });
    });
});
define("spec/indexSpec", ["require", "exports", "src/index"], function (require, exports, index_2) {
    "use strict";
    exports.__esModule = true;
    describe('Plugin Container', function () {
        var plugins;
        var plugin1;
        var plugin2;
        var str;
        var output;
        it('init', function () {
            plugin1 = {
                name: 'p1',
                priority: 1,
                execute: function (input) {
                    return input.replace(/blabla/gi, 'loremipsum');
                }
            };
            plugin2 = {
                name: 'p2',
                priority: 2,
                execute: function (input) {
                    return "avacadabra" + input + "flumflumblablasrpic";
                }
            };
            str = 'hello world blabla world';
            plugins = new index_2.PluginContainer();
            str = 'hello world blabla world';
            output = plugins.executeAll(str);
            expect(output).toBe(str);
            plugins.install(plugin1);
            output = plugins.executeAll(str);
            expect(output).toBe('hello world loremipsum world');
            plugins.uninstall(plugin1);
            output = plugins.executeAll(str);
            expect(output).toBe(str);
            plugins.install(plugin2);
            plugins.install(plugin1);
            output = plugins.executeAll(str);
            expect(output).toBe('avacadabrahello world loremipsum worldflumflumloremipsumsrpic');
            plugins.uninstall(plugin1);
            plugins.uninstall(plugin2);
            output = plugins.executeAll(str);
            expect(output).toBe(str);
            plugins.install(plugin1);
            plugins.install(plugin2);
            output = plugins.executeAll(str);
            expect(output).toBe('avacadabrahello world loremipsum worldflumflumloremipsumsrpic');
        });
    });
});
define("plugin-container", ["require", "exports", "src/index"], function (require, exports, index_3) {
    "use strict";
    exports.__esModule = true;
    exports.PluginContainer = index_3.PluginContainer;
});
//# sourceMappingURL=plugin-container-amd.js.map