"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../src/index");
describe('errors in plugins', () => {
    let container;
    function createContainerAndInstallPlugins(config) {
        container = index_1.create(config);
        container.install({
            name: 'thrower plugin',
            priority: 2,
            execute(input) {
                input.badPluginGreets = 'helloFromBadPlugin';
                throw Error('thrower!');
            },
        });
        container.install({
            name: 'good plugin',
            priority: 3,
            execute(input) {
                input.goodPluginGreets = 'helloFromGoodOne';
                return input;
            },
        });
        container.install({
            name: 'good firstPlugin',
            priority: 1,
            execute(input) {
                input.firstPlugin = 'firstPluginGreet';
                return input;
            },
        });
    }
    it('by default, plugins throwing exceptions should allow the rest of plugins to execute', () => {
        createContainerAndInstallPlugins();
        const obj = {};
        try {
            const result = container.executeAll(obj);
            expect(obj.goodPluginGreets).toBe('helloFromGoodOne');
            expect(obj.firstPlugin).toBe('firstPluginGreet');
            expect(obj.badPluginGreets).toBe('helloFromBadPlugin');
        }
        catch (error) {
            fail(error);
        }
    });
    it('onErrorPolicy==break should not throw but should now allow following plugins to execute', () => {
        createContainerAndInstallPlugins({ onErrorPolicy: 'break' });
        const obj = {};
        try {
            const result = container.executeAll(obj);
            expect(obj.firstPlugin).toBe('firstPluginGreet');
            expect(obj.goodPluginGreets).toBe(undefined);
            expect(obj.badPluginGreets).toBe('helloFromBadPlugin');
        }
        catch (error) {
            fail(error);
        }
    });
    it('onErrorPolicy==throw should throw ', () => {
        createContainerAndInstallPlugins({ onErrorPolicy: 'throw' });
        const obj = {};
        try {
            const result = container.executeAll(obj);
            fail('did not throw');
        }
        catch (error) {
            expect(obj.firstPlugin).toBe('firstPluginGreet');
            expect(obj.goodPluginGreets).toBe(undefined);
            expect(obj.badPluginGreets).toBe('helloFromBadPlugin');
        }
    });
    // TODO: a sync plugin that throws inside a settimeout
});
//# sourceMappingURL=errorsSpec.js.map