"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
class PluginContainer extends events_1.EventEmitter {
    constructor(config) {
        super();
        this.defaultConfig = {
            onErrorPolicy: 'continue',
        };
        this.plugins = [];
        this.config = Object.assign({}, this.defaultConfig, config || {});
    }
    // public static 'plugin-exception': 'plugin-exception' = 'plugin-exception';
    static create(config) {
        return new PluginContainer(config);
    }
    executeAll(input) {
        let value = input;
        this.plugins.some((p) => {
            try {
                this.emit('plugin-before-execute', { plugin: p, input: value });
                value = p.execute(value) || value;
                this.emit('plugin-success', { plugin: p, result: value });
            }
            catch (error) {
                if (this.config.onErrorPolicy === 'continue') {
                    this.emit('plugin-exception', { plugin: p, error, input: value });
                    return false;
                }
                else if (this.config.onErrorPolicy === 'break') {
                    this.emit('plugin-exception', { plugin: p, error, input: value });
                    return true; // we are using array.some - returning true will break the loop
                }
                else /*if (this.config.onErrorPolicy === 'throw') */ {
                    this.emit('plugin-exception', { plugin: p, error, input: value });
                    throw error;
                }
            }
        });
        return value;
    }
    install(plugin) {
        // TODO: emit before - after plugin install event
        plugin.priority = plugin.priority || PluginContainer.DEFAULT_PRIORITY;
        this.plugins.push(plugin);
        this.plugins = this.plugins.sort((a, b) => (a.priority < b.priority ? -1 : 1));
    }
    uninstall(plugin) {
        const name = this.getPluginName(plugin);
        this.plugins = this.plugins.filter(p => p.name !== name);
    }
    getPluginName(plugin) {
        return typeof plugin === 'string' ? plugin : plugin.name;
    }
}
PluginContainer.DEFAULT_PRIORITY = 5;
exports.PluginContainer = PluginContainer;
function create(config) {
    return PluginContainer.create(config);
}
exports.create = create;
//# sourceMappingURL=index.js.map